﻿//-----------------------------------------------------------------------------
// Game1.cs
//
// Kyoung Shin Park @ Dankook University (4/2/2011)
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using XNAGeometryObject;

namespace XNATexturePyramid
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // quad
        Pyramid py;
        Texture2D texture;

        // effect
        BasicEffect effect;
        Matrix projection;
        Matrix view;
        Matrix world;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;
        bool animationMode = false;  // false=no motion, true=animation
        bool fillMode = true;       // false=wireframe, true=solid
        bool shadingMode = true;    // false=flat shading, true=smooth shading 
        float yaw = 0.0f, pitch = 0.0f;
        float zoom = 1.0f;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            py = new Pyramid();
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            InitializeTransform();
            InitializeEffect();
            texture = Content.Load<Texture2D>("brick0");

            base.Initialize();
        }

        /// <summary>
        /// Initializes the transforms used by the game.
        /// </summary>
        private void InitializeTransform()
        {
            // world matrix
            world = Matrix.Identity;

            // view matrix
            view = Matrix.CreateLookAt(new Vector3(0, 0, 5.0f), Vector3.Zero, Vector3.Up);

            // projection matrix
            float aspectRatio = (float)GraphicsDevice.Viewport.Width / (float)GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, aspectRatio, 1.0f, 1000.0f);
        }

        /// <summary>
        /// Initializes the effect (loading, parameter setting, and technique selection)
        /// used by the game.
        /// </summary>
        private void InitializeEffect()
        {
            // Initialize Effect
            effect = new BasicEffect(GraphicsDevice);
            effect.Alpha = 1.0f;
            effect.PreferPerPixelLighting = true;

            effect.DiffuseColor = new Vector3(1.0f, 1.0f, 1.0f);
            effect.SpecularColor = new Vector3(1.0f, 0.0f, 0.0f);
            effect.SpecularPower = 16.0f;
            effect.AmbientLightColor = new Vector3(0.75f, 0.75f, 0.75f);

            effect.DirectionalLight0.Enabled = true;
            effect.DirectionalLight0.DiffuseColor = Vector3.One;
            effect.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(1.0f, -1.0f, -1.0f));
            effect.DirectionalLight0.SpecularColor = Vector3.One;

            effect.DirectionalLight1.Enabled = true;
            effect.DirectionalLight1.DiffuseColor = new Vector3(0.5f, 0.5f, 0.5f);
            effect.DirectionalLight1.Direction = Vector3.Normalize(new Vector3(-1.0f, -1.0f, 1.0f));
            effect.DirectionalLight1.SpecularColor = Vector3.One;

            effect.LightingEnabled = true;

            effect.World = world;
            effect.View = view;
            effect.Projection = projection;
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            HandleInput();

            base.Update(gameTime);
        }

        /// <summary>
        /// Determines which primitive to draw based on input from the keyboard
        /// or game pad.
        /// </summary>
        protected void HandleInput()
        {
            CheckGamePadInput();
            CheckKeyboardInput();
        }

        private void CheckGamePadInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);

            if (((currentGamePadState.Buttons.B == ButtonState.Pressed)) &&
                 (prevGamePadState.Buttons.B == ButtonState.Released))
            {
                fillMode = !fillMode;   // toggle fillmode
            }
            else if (((currentGamePadState.Buttons.A == ButtonState.Pressed)) &&
                      (prevGamePadState.Buttons.A == ButtonState.Released))
            {
                animationMode = !animationMode;
            }
            else if (((currentGamePadState.Buttons.X == ButtonState.Pressed)) &&
                      (prevGamePadState.Buttons.X == ButtonState.Released))
            {
                shadingMode = !shadingMode;
            }
        }

        private void CheckKeyboardInput()
        {
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            if (currentKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.W)
                && !prevKeyboardState.IsKeyDown(Keys.W))
            {
                fillMode = !fillMode;   // toggle wireframe/fillmode
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Space)
                && !prevKeyboardState.IsKeyDown(Keys.Space))
            {
                animationMode = !animationMode;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.S)
                && !prevKeyboardState.IsKeyDown(Keys.S))
            {
                shadingMode = !shadingMode;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.X)
                && !prevKeyboardState.IsKeyDown(Keys.X))
            {
                yaw += 5.0f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Y)
                && !prevKeyboardState.IsKeyDown(Keys.Y))
            {
                pitch -= 5.0f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Z)
                && !prevKeyboardState.IsKeyDown(Keys.Z))
            {
                zoom -= 0.1f;
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            if (animationMode) yaw += 0.8f;
            world = Matrix.CreateRotationX(MathHelper.ToRadians(pitch)) * Matrix.CreateRotationY(MathHelper.ToRadians(yaw)) * Matrix.CreateTranslation(0, 0, zoom);
            effect.World = world;
            effect.Texture = texture;
            effect.TextureEnabled = true;

            // effect
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Apply();
                // Draw the pyramid
                GraphicsDevice.DrawUserPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    py.Vertices, 0, py.Vertices.Length/3);
            }

            base.Draw(gameTime);
        }
    }
}
